/*
	Altera parallel port I/F Driver DLL Version 1.0

	Copyright(c) 2002 Tatsuyuki Satoh

	Altera BitBlaster
	I/O𒼐ڂ̂߁AWindowsNT/200/XPłgiveio.sysKv

	PDoE_XL(JTAG)[h}

	D0    -> TCK
	D1    -> TMS
	D5    -> PROBLE SENSE(to ACK)
	D6    -> TDI
	D7    -> PROBE SENSE with TVCC (to ACK)
	ERROR <- TargetVCC
	BUSY  <- TDO
	PR    <- PROBLE SENSE with TVCC(from D7)
	ACK   <- PROBLE SENSE(from D5)
	AFD   -> -EBABLE (TCK,TMS,TDI ebable)

	QDVAX[u[h}

	D0    -> CCLK
	D1    -> PROGRAM
	D5    -> PROBLE SENSE(to ACK)
	D6    -> DIN
	ERROR <- TargetVCC
	BUSY  <- DONE
	ACK   <- PROBLE SENSE(from D5)
	AFD   -> -EBABLE (TCK,TMS,TDI ebable)

	Export DLL Functions

	JTAG BounteryScan
		int jtag_version(void)
		int jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
		int jtag_power(void)
		int jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
		void jtag_end(void)

	CCLK SerialSlave
		int ss_version(void)
		int ss_init(DWORD port_address,DWORD baudrate,char *probe_name);
		int ss_power(void);
		int ss_tx(const BYTE *data,DWORD size);
		int ss_program(BYTE level);
		int ss_done(void);
		void ss_end(void);

	note:

		RpC̓t[̃RpCAMINGWgpĂ܂B

	Histry
		2001. 8. 6 1st release

*/
#include <windows.h>
#include <stdio.h>

// DLL Export prefix
#define DLLEXPORT __declspec(dllexport)
#ifdef __MINGW32__
#define DLLAPI
#else
#define DLLAPI WINAPI
#endif

/* ---------------------------------------------------------- */

//ph^egptO@^@h^nAhX
static DWORD par_port;

#define MASK_DEF 0x00
#define MASK_TCK 0x01 /* D0 */
#define MASK_TMS 0x02 /* D1 */
#define MASK_TDI 0x40 /* D6 */

#define MASK_TDO   0x80 /* BUSY */
#define MASK_TVCC  0x08 /* ERROR */
#define MASK_SE0   0x40 /* ACK */
#define MASK_SE1   0x20 /* PE  */

// JTAG out 1clock
#define PAR_JTAG(TMS,TDI) {\
	_outp(par_port,(TMS?MASK_TMS:0) | (TDI?MASK_TDI:0) | MASK_DEF); \
	_outp(par_port,(TMS?MASK_TMS:0) | (TDI?MASK_TDI:0) | MASK_DEF | MASK_TCK); \
}

// JTAG output enable
#define PAR_ENABLE(ON) _outp(par_port+2,(ON?0x02:0x00))

// JTAG TDO sense
#define PAR_TDO() ((~_inp(par_port+1))&MASK_TDO)
// VCC sense
#define PAR_VCC() (_inp(par_port+1)&MASK_TVCC)
// probe sense
#define PAR_PR0() (_inp(par_port+1)&MASK_SE0)
// probe sense
#define PAR_PR1() (_inp(par_port+1)&MASK_SE1)

DLLEXPORT DWORD DLLAPI jtag_version(void)
{
	DWORD mjr = 1;
	DWORD mnr = 0;
	return (mjr<<16) + mnr;
}

//p|[g̏
DLLEXPORT int DLLAPI jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	HANDLE h;

	/* call GIVEIO to direct I/O access for WindowsNT/2000/XP */
	h = CreateFile("\\\\.\\giveio", GENERIC_READ, 0, NULL,
					OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(h != INVALID_HANDLE_VALUE)
		CloseHandle(h);

	par_port = port_address==0 ? 0x378 : port_address;

	if( probe_name != NULL)
		sprintf(probe_name,"Altera BitBlaster ISP Cable (JTAG),IO Base=%04X",(int)port_address);

	// probe sense
	_outp(par_port,0x00);
	if( PAR_PR0() )
	{
		printf("No Probe\n");
		return 2;
	}
	_outp(par_port,0x20);
	if( !PAR_PR0() )
	{
		printf("No Probe\n");
		return 2;
	}

	// JTAG output enable
	PAR_ENABLE(1);

	return 0;
}

//d`FbN
DLLEXPORT int DLLAPI jtag_power(void)
{
	if(!PAR_VCC())
		return 0;

	_outp(par_port,0x00);
	if( PAR_PR1())
		return 0;

	_outp(par_port,0x80);
	if( !PAR_PR1())
		return 0;

	return 1;
}

//is`fɂ鑗M
DLLEXPORT int DLLAPI jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
{
	int c;

	// clear outbuf
	if(obuf)
		for(c=0;c<(drsize+7)/8;c++)
			obuf[c] = 0;

	for(c=0;c<drsize;c++)
	{
		int tdi = ibuf && (ibuf[c/8]&(0x80>>(c&7))) ? 1 : 0;
		int tms = (c==drsize-1) ? tms_e : tms_s;
		// out 1 TCK
		PAR_JTAG(tms,tdi);
		// TDO get
		if( obuf && PAR_TDO() )
			obuf[c/8] |= 0x01<<(c&7);
	}
	return 0;
}

DLLEXPORT void DLLAPI jtag_end(void)
{
}

/*-------------------------------------------------------------------------------*/
#define CF_DIN  0x01
#define CF_CCLK 0x02
#define CF_PROGRAM 0x04
#define CF_BASE (0x10|CF_PROGRAM)

#define PAR_DONE() ((_inp(par_port+1)>>4)&1)

//return version
DLLEXPORT int DLLAPI ss_version(void)
{
	DWORD mjr = 1;
	DWORD mnr = 0;
	return (mjr<<16) + mnr;
}

DLLEXPORT int DLLAPI ss_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	if( probe_name != NULL)
		sprintf(probe_name,"Altera BitBlaster ISP Cable (CCLK),IO Base=%04X",(int)port_address);

	return jtag_init(port_address,baudrate,NULL);
}

DLLEXPORT int DLLAPI ss_power(void)
{
	return jtag_power();
}

DLLEXPORT int DLLAPI ss_tx(const BYTE *data,DWORD size)
{
	int pos;

	// PubN̑Mf[^쐬
	for(pos=0 ; pos<size ; pos++)
	{
		BYTE value  = data[pos];

		BYTE outval;
		outval = (value&0x80) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x40) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x20) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x10) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x08) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x04) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x02) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x01) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
	}
	return 0;
}

DLLEXPORT int DLLAPI ss_program(BYTE level)
{
	_outp(par_port,0x10);
	_outp(par_port,0x10);
	_outp(par_port,CF_BASE);
	return 0;
}

DLLEXPORT int DLLAPI ss_done(void)
{
	return PAR_DONE();
}

DLLEXPORT void DLLAPI ss_end(void)
{
}
